<?xml version="1.0" encoding="UTF-8"?>

<!--
    Created on : September 3, 2007, 8:55 PM
    Author     : Jody Gustafson
    Version    : 20071010
    Description: Base stylesheet that implements all basic elements for MPML.
-->

<xsl:stylesheet version="1.1"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:mp="http://www.worldtreesoftware.com/mpml"
  exclude-result-prefixes="mp html">
    
  <xsl:output method="html" doctype-public="-//W3C//DTD HTML 4.01 Transitional//EN"/>
  <!--<xsl:output method="html" doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"/>-->
  <xsl:strip-space elements="*"/>

  <!-- Stylesheet parameters set by transformer -->
  <xsl:param name="now"/>
  <xsl:param name="version"/>

  <!-- Global variables -->
  <xsl:variable name="debug" select="string-length(/mp:mpml/@debug) != 0"/>
  <!--<xsl:param name="debug" select="true()"/>-->
  
  <!-- Display the ModPro comment at the top of the file -->
  <xsl:template match="/">
    <xsl:comment>Generated by ModPro Compiler version <xsl:value-of select="$version"/> on <xsl:value-of select="$now"/>. http://www.WorldTreeSoftware.com</xsl:comment>
    <xsl:apply-templates />
  </xsl:template>

  <!-- Root element is always mpml -->
  <xsl:template match="/mp:mpml">
    <xsl:apply-templates select="html"/>
  </xsl:template>

  <!-- Implement referenced module -->
  <xsl:template match="mp:module[@ref]">
    <xsl:variable name="name" select="@ref"/>
    <xsl:apply-templates select="//mp:module[@name=$name]"/>
  </xsl:template>

  <!-- Implement module -->
  <xsl:template match="mp:module">
    <xsl:if test="$debug"><xsl:call-template name="mp:debugCommentBegin"/></xsl:if>
    <xsl:apply-templates/>
    <xsl:if test="$debug"><xsl:call-template name="mp:debugCommentEnd"/></xsl:if>
  </xsl:template>

  <!-- Adds a comment to the output -->
  <xsl:template match="mp:comment">
    <xsl:comment><xsl:value-of select="."/></xsl:comment>
  </xsl:template>

  <!-- Content that adds the current date -->
  <xsl:template match="mp:timestamp">
    <xsl:value-of select="$now"/>
  </xsl:template>

  <!-- Debugging templates -->
  <xsl:template name="mp:debugCommentBegin">
    <xsl:comment><xsl:value-of select="concat('BEGIN ', name(), ' ', @name)"/></xsl:comment>
  </xsl:template>
  <xsl:template name="mp:debugCommentEnd">
    <xsl:comment><xsl:value-of select="concat('END ', name(), ' ', @name)"/></xsl:comment>
  </xsl:template>
  
  <!-- Output any text that isn't inside an element -->
  <xsl:template match="text()">
    <xsl:value-of select="."/>
  </xsl:template>

  <!-- Output any elements that aren't mpml elements (assume they are valid html) -->
  <!-- TODO: Consider making this only match on the html namespace -->
  <xsl:template match="*">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>